import abc


class AccrocchioObserver(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def on_accrocchio(self):
        pass  # pragma: no cover

    @abc.abstractmethod
    def reset(self):
        pass  # pragma: no cover


ACCROCCHIO_OBSERVERS = {}


def add_accrocchio_observer(observer: AccrocchioObserver, accrocchio_name: str='accrocchio') -> None:
    observers = ACCROCCHIO_OBSERVERS.get(accrocchio_name)
    if not observers:
        observers = []
        ACCROCCHIO_OBSERVERS[accrocchio_name] = observers
    observers.append(observer)


def reset(accrocchio_name: str=None):
    for k, x in ACCROCCHIO_OBSERVERS.items():
        if accrocchio_name and k != accrocchio_name:
            continue
        for y in x:
            y.reset()



import os
import sys

from setuptools import setup, find_packages


major, minor1, minor2, release, serial = sys.version_info

readfile_kwargs = {"encoding": "utf-8"} if major >= 3 else {}


def readfile(filename):
    with open(filename, **readfile_kwargs) as fp:
        contents = fp.read()
    return contents


def get_packages(path):
    out = [path]
    for x in find_packages(path):
        out.append('{}/{}'.format(path, x))
    
    return out

packages = get_packages('accrocchio')
setup(name='accrocchio',
      version='0.1.5',
      description='Accrocchio is a library to mark and being notified of smelly code (a.k.a, "accrocchio").',
      long_description=readfile('README.rst'),
      url='https://github.com/fcracker79/accrocchio',
      author='fcracker79',
      author_email='fcracker79@gmail.com',
      license='MIT',
      packages=packages,
      install_requires=readfile(os.path.join(os.path.dirname(__file__), "requirements.txt")),
      zip_safe=False,
      test_suite="test",
      classifiers=[
          'Development Status :: 4 - Beta',
          'Intended Audience :: Developers',
          'Topic :: Software Development',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 3.5',
      ],
      keywords='accrocchio hinting linting qa'
      )
